<?php
/**
 * Clase que define las caractersticas particulares del driver oci8
 * Sobre la portabilidad, este driver por defecto activa DB_PORTABILITY_LOWERCASE
 * y DB_PORTABILITY_DELETE_COUNT.
 *
 * @package gvHIDRA
 */
include_once('igep/include/igep_bd/IgepDBMS.php');

class IgepDBMS_oci8 extends IgepDBMS {

  /**
   * Modifica, si procede, los parmetros de la conexin.
   * @param mixed dsn que utiliza pear:db para la conexin
   * @return mixed devuelve el dsn modificado
   */
  function preConexion($p_dsn){
  	if ($p_dsn['database'] != '')
		throw new Exception('El parmetro "database" no se usa para oracle');
    return $p_dsn;
  }

  /**
   * Acciones realizadas:
   * - Fija el formato por defecto de las fechas
   * - Fijar el idioma espaol (no porque de momento ya se ha configurado el servidor en espaol)
   * - Registrar la sesin en oracle
   * @param conexion recibe una conexin establecida
   */
  function postConexion($p_conexion){
    $query = 'alter session set nls_date_format=\'dd/mm/yyyy hh24:mi:ss\'';
    $result = $p_conexion->exec($query);
    if (PEAR::isError($result)){
		throw new Exception('Error al fijar variables de sesin nls_date_format');
    }
    $query = 'alter session set nls_numeric_characters=\'.,\'';
    $result = $p_conexion->exec($query);
    if (PEAR::isError($result)){
		throw new Exception('Error al fijar variables de sesin nls_numeric_characters');
    }
    // Registrar sesion
    $aplicacion=IgepSession::dameAplicacion();
    // Lo ideal seria coger el modulo URL que estoy ejecutando, en vez del script_name
    $mod = strtoupper(IgepSession::dameUsuario()).'@'.$_SERVER['SCRIPT_NAME'];
	$query = "begin dbms_application_info.set_module( '$aplicacion', '$mod' ); end;";
    $result = $p_conexion->exec($query);
    if (PEAR::isError($result)){
            echo 'Error al registrar la sesin: '.$query;
            return;
    }
 }		

  /**
   * Indica los carcteres usados para esta conexin (separador decimal y de miles)
   *
   * @access public
   * @static
   * @param mixed dsn que utiliza pear:db para la conexin
   * @return mixed array asociativo con entrada 'DECIMAL' y 'GROUP'
   */
  function caracteresNumericos($p_dsn){
  	return array('DECIMAL'=>'.','GROUP'=>'');
  }

  /**
   * Indica la mscara de fechas utilizada para esta conexin
   *
   * @access public
   * @static
   * @param mixed dsn que utiliza pear:db para la conexin
   * @return string que indica la mascara de fechas utilizada.
   */
  function mascaraFechas($p_dsn){
    return 'j/n/Y';
  }

  /**
   * Devuelve la cadena SQL que permite obtener el valor de una secuencia de BD
   *
   * @access public
   * @static
   * @param string sequence cadena que contiene el nombre de la secuencia.
   * @return string
   */
  function obtenerSecuenciaBD($sequence){
    return 'SELECT '.$sequence.'.NextVal as "nextval" from DUAL';
  }

  /**
  * Devuelve si el objeto de error es debido a que la(s) fila(s) no se puede(n) bloquear
  * [Native code: 54[Native message: ORA-00054: resource busy and acquire with NOWAIT specified
  * 
  * @access public
  * @param result objeto error de IgepError
  * @return boolean
  */
  function isLocked($result){
    return (strpos($result,'[Native code: 54[')!==false);
  }

  /**
  * Devuelve la cadena que se debe utilizar para escapar la contrabarra
  * 
  * En Oracle no utilizamos dicha cadena, en Postgres y MySql si 
  * 
  * @access public
  * @return string
  */
  public function backSlashScape(){

    return "\\";
  }
}

?>