<?php
/**
 * Validacion de usuarios: configuracion personalizable por el usuario
 * Clase base para crear nuevos metodos de autenticacion para gvHIDRA
 *
 * @package gvHIDRA
 */

require_once 'Auth.php';
include_once 'Auth/Container.php';

abstract class gvhBaseAuth extends Auth_Container {

	/**
	 * Metodo para llamar desde aplicaciones, en la autenticacion inicial
	 * 
	 * Devuelve cadena vacia si todo va bien, o texto si error
	 */
	static function autenticate($p_apli) {}

	/**
	 * Metodo para autenticar usuario 
	 */
	function fetchData($username, $password, $isChallengeResponse=false) {}

	/**
	 * Comprueba que la sesin tiene toda la informacin necesaria para gvHIDRA
	 * Si falta algo se produce una excepcion
	 * 
     * Informacin usada por gvHidra (se puede modificar la ubicacin aunque habria que
     * cambiar el mtodo correspondiente en la clase igep/include/ComunSession.php):
     *
     * $_SESSION['LINT']['daplicacion'] --> nombre de la aplicacin
     * 
     * $_SESSION['LINT']['modulos'] --> matriz de mdulos asignados al usuario con las siguientes columnas:
     *      $_SESSION['LINT']['modulos']['P_MODIFICA']['valor'] --> valor del mdulo
     *      $_SESSION['LINT']['modulos']['P_MODIFICA']['descrip']--> descripcin del mdulo
     *
     * $_SESSION['LINT']['rolusuar'] --> role del usuario
     * 
     * (siendo, 'LINT' y 'P_MODIFICA' ejemplos de aplicacin y mdulo, respectivamente)
     * 
	 * @param mixed p_sesion
	 * @param string p_apli 
	 */
	function checkData($p_sesion, $p_apli)
	{
		if (!isset($p_sesion[$p_apli]['usuario']['usuario']))
			throw new Exception('La sesin no tiene el login del usuario conectado');
		if (!isset($p_sesion[$p_apli]['usuario']['nombre']))
			throw new Exception('La sesin no tiene el nombre del usuario conectado');

		if (!isset($p_sesion[$p_apli]['rolusuar']))
			throw new Exception('La sesin no tiene el perfil del usuario conectado');
		if (!isset($p_sesion[$p_apli]['modulos']))
			throw new Exception('La sesin no tiene los mdulos del usuario conectado');
		if (!isset($p_sesion[$p_apli]['daplicacion']))
			throw new Exception('La sesin no tiene el titulo de la aplicacin');

		if (!isset($p_sesion['validacion']['bd']))
			throw new Exception('La sesin no tiene la fuente de autenticacin');
		if (!isset($p_sesion['validacion']['server']))
			throw new Exception('La sesin no tiene el servidor www');
	}

	/**
	 * Hace una redireccion al inicio de la aplicacion
	 * pasando como argumento la direccion del metodo validacion::valida a usar
	 * Tambien se pasa la pagina actual para volver a ella cuando hacemos un logout
	 * Va en consonancia con openApp_inc.php
	 */
	function open($url)
	{
		$login = 'http'.(isset($_SERVER['HTTPS'])?'s':'').'://'.$_SERVER['SERVER_NAME'].':'.$_SERVER['SERVER_PORT'].$_SERVER['PHP_SELF'];
		session_write_close();
		header('location: ./openApp.php?option='.urlencode($url).'&login='.urlencode($login));
	}
}

?>
